@@ -80,6 +80,11 @@ class TestCase:
         """
         raise NotImplementedError
 
+    @property
+    def max_duration(self):
+        """Returns the maximum time in sec that an instance of this test may take to run"""
+        raise NotImplementedError
+
     def setup(self):
         """Defines setup for a single execution of this test."""
         raise NotImplementedError
@@ -111,6 +116,7 @@ class TestRunner:
         self._lock = threading.Lock()
         self.test_cls = test_cls
         self.test_cls_kwargs = test_cls_kwargs
+        self.test_instance = test_cls(**test_cls_kwargs)
         self.rate = rate  # num tests per sec
         self.active_tests = 0
         self.successes = 0
@@ -178,7 +184,7 @@ class TestRunner:
         start_time = time.time()
 
         rate_of_fire = 1.0 / self.rate
-        max_duration = 180
+        max_duration = self.test_instance.max_duration
         workers = rate_of_fire * max_duration
 
         executor = ThreadPoolExecutor(max_workers=workers)
@@ -278,7 +284,7 @@ class ChildProcess(multiprocessing.Process):
         results = []
         for pt in self.perf_testers:
             # Instantiate the test class w/ kwargs so we can get the unique name/summary properties
-            test = pt.test_cls(**pt.test_cls_kwargs)
+            test = pt.test_instance
             test_name = f"{str(test)} -- {test.summary}"
             results.append(
                 {
@@ -290,20 +296,7 @@ class ChildProcess(multiprocessing.Process):
             )
         self.result_queue.put(results)
 
-    def run(self):
-        """
-        The logic running within each child process.
-
-        Takes care of:
-        1. running session startup for each test case class
-        2. launching a TestRunner for that TestCase
-        3. initiating the start of the TestRunners (only if their startup worked)
-        4. trigger stop of the TestRunners
-        5. running session shutdown for each test case class
-        6. sending results back to the main process on result_queue
-        """
-        signal.signal(signal.SIGINT, signal.SIG_IGN)  # worker procs ignore sigint
-
+    def _init_test_runners(self):
         # Run startup once per test class
         for test_config in self.config["tests"]:
             test_cls = test_config["test_class"]
@@ -324,6 +317,29 @@ class ChildProcess(multiprocessing.Process):
                 self.perf_testers.append(pt)
                 total_rate_this_proc += pt.rate
 
+        return total_rate_this_proc
+
+    def run(self):
+        """
+        The logic running within each child process.
+
+        Takes care of:
+        1. running session startup for each test case class
+        2. launching a TestRunner for that TestCase
+        3. initiating the start of the TestRunners (only if their startup worked)
+        4. trigger stop of the TestRunners
+        5. running session shutdown for each test case class
+        6. sending results back to the main process on result_queue
+        """
+        signal.signal(signal.SIGINT, signal.SIG_IGN)  # worker procs ignore sigint
+
+        try:
+            total_rate_this_proc = self._init_test_runners()
+        except Exception:
+            log.exception("Child process hit error during startup")
+            self.result_queue.put("STARTUP_FAILED")
+            return
+
         log.info("Sending 'startup done' to main proc and waiting for it to tell me to start...")
         self.result_queue.put("STARTUP_DONE")
 
@@ -358,6 +374,17 @@ class ChildProcess(multiprocessing.Process):
 class Manager:
     """Handles running multiple TestRunners, splitting them across ChildProcesses."""
 
+    @staticmethod
+    def _validate_test_cls(test_cls, test_cls_kwargs):
+        instance = test_cls(**test_cls_kwargs)
+
+        try:
+            instance.max_duration
+        except NotImplementedError:
+            raise AttributeError(
+                f"{type(instance)} must have max_duration property defined"
+            ) from None  # avoid printing the original chained error
+
     def __init__(self, config, duration, debug=False):
         # TODO: real config management
         self.config = config
@@ -374,12 +401,15 @@ class Manager:
         sys.path.insert(0, os.getcwd())
 
         # Map the test class string in the config into an actual imported class
+        log.info("Validating test classes...")
         for test in self.config["tests"]:
             test_cls = test["test_class"]
+            test_cls_kwargs = test.get("kwargs", {})
             imported_cls = locate(test_cls)
             if not imported_cls:
                 raise ValueError(f"Unable to import test class '{test_cls}'")
             test["test_class"] = imported_cls
+            self._validate_test_cls(imported_cls, test_cls_kwargs)
 
         # Remove freshly added cwd from path
         sys.path.pop(0)
@@ -477,14 +507,22 @@ class Manager:
         # wait for the 'startup' to finish
         results = []
         log.info("Waiting to receive 'startup done' from child procs...")
+        failed = False
         while len(results) < len(self.child_procs):
             try:
                 result = self.result_queue.get(block=False)
-                if result != "STARTUP_DONE":
-                    raise Exception("Expected a STARTUP_DONE result, got something else")
+                if result == "STARTUP_FAILED":
+                    log.debug("Startup for a child process failed")
+                    failed = True
+                elif result != "STARTUP_DONE":
+                    log.debug("Expected a STARTUP_DONE result, got something else")
+                    failed = True
                 results.append(result)
             except queue.Empty:
-                time.sleep(1)
+                time.sleep(0.1)
+
+        if failed:
+            raise Exception("Startup of child processes failed")
 
         # Now trigger the actual run
         log.info("Starting...")
